; ****************************************************
;        Software controlled RC oscillator
;
; ....How to convert a AVR/1200 into a 555....
;
; This example is a demonstrator of the powerful
; hw/sw co-simulation capabilities of VMLAB. It shows:
;
;  - Mixed mode analog/digital simulation.
;  - Interactive elements (button driven key).
;  - A software-in-the-loop system.
;  - Interrutps.
;
; ****************************************************

.include "..\include\1200def.inc"

;====================================================
; Reset and interrupt vectors. It is convenient (though
; not mandatory) to place labels identifying the
; interrupt vectors to better trace them later.
;
reset:
   rjmp start            ; Reset vector
int0_vect:
   rjmp int0_ser         ; INT0 vector
ovf0_vect:
   reti                  ; Timer0 overflow (unused)
comp_vect:
   reti                  ; Comparator (unused)

;====================================================
; INT0 service. While in this routine the RC circuit
; won't oscillate. Press button '0' in Control Panel
; to activate the interrupt. It will produce a pulse
; train in PB2 port
;
int0_ser:
   ldi r16, 255
loop:
   sbi PORTB, PB2        ; Produces 255
   nop                   ; pulses over
   nop                   ; PB2
   nop                   ;
   cbi PORTB, PB2
   nop
   dec r16
   brne loop
   reti

;=====================================================
; Initialization
;
start:
   sbi DDRB,  PB0        ; PB0 -> output
   sbi DDRB,  PB1        ; PB1 -> output
   sbi DDRB,  PB2        ; PB2 -> output
   sbi PORTD, PD2        ; PD2 -> input pullup
   nop
   nop                   ; Some delay
   nop                   ; to discharge capacitor
   nop                   ;
   nop
   cbi DDRB, PB1         ; PB1 -> input
   ldi r16, (1<<INT0)
   out GIMSK, r16        ; Enable INT0 interrupt
   sei                   ; Global interrupts Enable

; Main oscillator control loop
;
forever:
   sbi  PORTB, PB0       ; Set PB0
wait1:
   sbis PINB,  PB1       ; Wait till PB1 is 1
   rjmp wait1

   cbi  PORTB, PB0       ; Clear PB0
wait2:
   sbic PINB,  PB1       ; Wait till PB1 is 0
   rjmp wait2
rjmp forever
