;*****************************************************************
;*  PWM based sinewave generator, table based
;**************************************************************************

.include "..\include\2313def.inc"

; Registers definitions
;
.def tmp = r16
.def angle = r17      ; Index to hold the sine phase angle (0 to 127)

;**************************************************************************
; Reset and interrupt vectors handlers
;
.cseg
.org $00
reset_hnd:
    rjmp start        ;
.org OVF1addr
tim1_hnd:
    rjmp tim1_ovf

;**************************************************************************
; Timer 1 interrupt handler
;
tim1_ovf:
   inc angle
   andi angle, $7F                ; truncates to 7 bits (0 - 127)
   ldi ZL,low(sine_tbl*2)         ; Get the table address and
   ldi ZH,high(sine_tbl*2)        ; add the angle phase. Result
   add ZL,angle                   ; will be in R0 after calling 'lpm'
   clr tmp                        ; lpm uses 16 bits index (Z)
   adc ZH, tmp                    ; Z index is the pointer to the
   lpm                            ; sine_tbl value for the angle phase

   clr tmp
   out OCR1AH, tmp                ; Reload Timer 1 compare value.
   out OCR1AL,R0                  ; Necessary to reload both registers
   reti

;**************************************************************************
; Reset handler. Initalizes port and Timer 1, and stay in a endless loop
;
start:
   sbi DDRB, PB3                  ; Set pin PB3 as output (is OC1 pin)
   ldi tmp,low(RAMEND)
   out SPL,tmp                    ; Init Stack pointer

   ldi tmp,(1<<TOIE1)
   out TIMSK,tmp                  ; Enable Timer1_ovf interrupt

   ldi tmp,(1<<PWM10)+(1<<COM1A1) ; Set Timer 1 in PWM mode
   out TCCR1A,tmp                 ; 8 bit PWM not reverse (Fck/510)
   ldi tmp,(1<<CS10)
   out TCCR1B,tmp                 ; prescaler = 1

   clr angle                      ; Start with phase = 0

   sei                            ; Enable interrupts

main:
   rjmp  main  ; Infinite loop: Timer1 interrupt will handle all

;*************************** SINE TABLE ***************************************
; Samples table : one period sampled on 128 samples and
; quantized on 7 bit
;
sine_tbl:
.db 0,0
.db 1,1
.db 2,3
.db 4,6
.db 7,9
.db 10,12
.db 14,16
.db 18,21
.db 23,25
.db 28,31
.db 33,36
.db 39,42
.db 45,48
.db 51,54
.db 57,60
.db 64,67
.db 70,73
.db 76,79
.db 82,85
.db 88,91
.db 94,96
.db 99,102
.db 104,106
.db 109,111
.db 113,115
.db 117,118
.db 120,121
.db 123,124
.db 125,126
.db 126,127
.db 127,127
.db 127,127
.db 127,127
.db 126,126
.db 125,124
.db 123,121
.db 120,118
.db 117,115
.db 113,111
.db 109,106
.db 104,102
.db 99,96
.db 94,91
.db 88,85
.db 82,79
.db 76,73
.db 70,67
.db 64,60
.db 57,54
.db 51,48
.db 45,42
.db 39,36
.db 33,31
.db 28,25
.db 23,21
.db 18,16
.db 14,12
.db 10,9
.db 7,6
.db 4,3
.db 2,1
.db 1,0
.db 0,0
.db 0,0



