; ********************************************
; STEP03.ASM
;
; In this step we have connected a pulse generator to
; the NMI, let's implement some NMI service routine.
; For example, a routine that produces 4 pulses
; in the PB0 port
;
; As always, firstly browse the code and afterwards
; press the the green light button.

.VERS "ST6210"
.ROMSIZE 2

.INPUT "regdef.h"  ; Registers include file

; RAM variables
; -------------
;

VAR_1   .DEF    084H,0FFH,0FFH,M
VAR_2   .DEF    085H,0FFH,0FFH,M
PB_COPY .DEF    086H,0FFH,0FFH,M
A_SAVE  .DEF    087H,0FFH,0FFH,M

.ORG    0880H

; Reset point. Variables and ports initialization
; -----------------------------------------------
;

INIT
   LDI PA,      00001111b
   LDI PADIR,   00001111b ; PA0-PA3 are outputs
   LDI PAOR,    00001111b

   LDI PB,      00000001b ; PB0 is output, used
   LDI PBDIR,   00000001b ; in the NMI routine.
   LDI PBOR,    00000001b ; PB1-7 are input pullups.
   LDI PB_COPY, 00000001b ; Copy register to SET/RES

   LDI VAR_1, 0
   LDI VAR_2, 255
   RETI

;                   * = TO DO's
;                   -----------

MAIN
   LDI WDT, 0FFH  ; Watchdog refresh
   CALL SUB_1     ;
   INC VAR_1
   DEC VAR_2      ; Some doing-nothing stuff
   LD A, VAR_1
   LD PA, A       ; Just transfer the VAR_1 to port
   JP MAIN

SUB_1
   CALL SUB_2
   RET

SUB_2
   NOP
   RET

; NMI service routine: produce 4 pulses in PB0
; --------------------------------------------
;

NMI_SERV
   LD A_SAVE, A      ; Save accumulator
   LDI X, 4          ; Load nr. of pulses
LOOP
   RES 0, PB_COPY
   LD A, PB_COPY     ; Loop to generate the pulses
   LD PB, A          ; Use the SET/RES on copy register
   SET 0, PB_COPY    ; and transfer later
   LD A,  PB_COPY
   LD PB, A
   DEC X
   JRNZ LOOP
   LD A, A_SAVE      ; Restore accumulator
   RETI

; Interrupt vectors and RESET
; ----------------------------
;
.ORG 0FF0H

   NOP
   RETI
   NOP
   RETI
   NOP
   RETI
   NOP
   RETI

.ORG 0FFCH

   JP NMI_SERV ; NMI interrupt service
   JP INIT     ; RESET vector
.END

