; ********************************************
; STEP06.ASM
;
; An any time the execution is paused, either by a breakpoint; the
; Stop button, or a Run-time warning,  you will see a second line of buttons
; on the right of the code. A button in a line means 'traced' and possible
; to x-reference. X-reference to what ?.
;
; Another Trace/Prio channel shows up in the Scope with 2 items:
; 1) the instructions timeframe; and 2) the task priority. In ST6,
; the task priority may have tree values: Normal; Interrupt and NMI,
; for which are 3 flag sets (See ST databook).
;
; The instruction timeframe boxes appair in different depth according
; to the stack level: a piece of code belonging to a subroutine
; called from the 'main' level is one level down, etc.

; You already have used the cursors; discover them now for a new use:
; cross reference. They work as well in the scope as in the code.
; Select a cursor; go to the code, and press a trace button... and see.

; Do not forget the 'todo's' (***)  on trace and cross - reference.
; See below.

.VERS "ST6210"
.ROMSIZE 2

.INPUT "regdef.h"  ; Registers include file

; Ram variables
; -------------
;
VAR_1   .DEF    084H,0FFH,0FFH,M
VAR_2   .DEF    085H,0FFH,0FFH,M
PB_COPY .DEF    086H,0FFH,0FFH,M
A_SAVE  .DEF    087H,0FFH,0FFH,M

.ORG    0880H

; Reset point. Variables and ports initialization
; -------------------------- ---------------------
;
INIT
   LDI PA,      00001111b
   LDI PADIR,   00001111b ; PA0-PA are outputs
   LDI PAOR,    00001111b

   LDI PB,      00000001b ; PB0 is output, used
   LDI PBDIR,   00000001b ;  in the NMI routine.
   LDI PBOR,    00000001b ; PB1-7 are input pullup.
   LDI PB_COPY, 00000001b ; Copy register to SET/RES

   LDI VAR_1, 0
   LDI VAR_2, 255
   RETI

;                   *** = TO DO's
;                   --------------
MAIN
   LDI WDT, 0FFH  ; ..
   CALL SUB_1     ;       .
   INC VAR_1      ;        .
   DEC VAR_2      ;       .
   LD A, VAR_1    ;        .
   LD PA, A       ;
   JP MAIN
                  ;
                  ; user !!

                  ; *** Locate a code part simultaneous
                  ; to half period of the simusoidal generator
SUB_1
   CALL SUB_2
   RET

SUB_2
   NOP
   NOP
   CALL SUB_3     ;
   RET

SUB_3
   NOP            ; See in the Scope the nesting effect of these calls
   NOP
   RET

; NMI service routine: produce 4 pulses in PB0
; --------------------------------------------
;
NMI_SERV
   LD A_SAVE, A      ; ***
                     ; *** Measure the time of the next 4 instructions
   LDI X, 4          ; *** From this one
LOOP                 ;
   RES 0, PB_COPY    ; *** (If you get 25.9 useg,at 8MHz,well done!!)
   LD A, PB_COPY     ; ***
   LD PB, A          ; *** Up to this one
   SET 0, PB_COPY    ;
   LD A,  PB_COPY    ; Loop to generate the pulses
   LD PB, A          ; Use the SET/RES on copy registers
   DEC X             ; and transfer later
   JRNZ LOOP
   LD A, A_SAVE      ; Restore acumulator
   RETI

; Interrupt vectors and RESET
; ----------------------------
;
.ORG 0FF0H

   NOP
   RETI
   NOP      ;
   RETI     ;
   NOP      ;
   RETI     ;
   NOP      ;
   RETI

.ORG 0FFCH

   JP NMI_SERV ; NMI interrupt service
   JP INIT     ; RESET vector
.END

